using System;

using Lazy;

using NUnit.Framework;

namespace Lazy.Example.EconomicAnalysis
{
	/// <summary>
	/// Descripcin breve de DemandTest.
	/// </summary>
	/// 
	[TestFixture]
	public class DemandTest
	{

		public const double PRICE_PRECISION = 0.005;

		[Test]
		public void TestKnownMarket()
		{
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.KnownMarket(10.00,1000);
			demand.PotentialMarketSize = 1000000;

			Assertion.AssertEquals ( 1000, demand.Quantity(  0.01) );
			Assertion.AssertEquals ( 1000, demand.Quantity(  9.99) );
			Assertion.AssertEquals ( 1000, demand.Quantity( 10.00) );
			Assertion.AssertEquals (    0, demand.Quantity( 10.01) );
			Assertion.AssertEquals (    0, demand.Quantity(100.00) );
		}

		[Test]
		public void TestLinearDemand() {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.PotentialMarketSize = 1000000;
			demand.MaximumPrice = 100.00;

			Assertion.AssertEquals (999900, demand.Quantity(  0.01) );
			Assertion.AssertEquals (990000, demand.Quantity(  1.00) );
			Assertion.AssertEquals (900000, demand.Quantity( 10.00) );
			Assertion.AssertEquals (500000, demand.Quantity( 50.00) );
			Assertion.AssertEquals (     0, demand.Quantity(100.00) );
			Assertion.AssertEquals (     0, demand.Quantity(200.00) );
		}

		[Test]
		public void TestElasticity() {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.KnownMarket(10.00,1000);
			demand.FixedElasticity = -1;
			
			Assertion.AssertEquals (  1000, demand.Quantity(10.00) );
			Assertion.AssertEquals (  1500, demand.Quantity( 5.00) );
			Assertion.AssertEquals (   500, demand.Quantity(15.00) );
			Assertion.AssertEquals (   250, demand.Quantity(17.50) );
			Assertion.AssertEquals (     0, demand.Quantity(20.00) );
			Assertion.AssertEquals (     0, demand.Quantity(30.00) );
			
		}

		[Test]
		public void TestInelasticity() {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.KnownMarket(10.00,1000);
			demand.FixedElasticity = -5;
			
			Assertion.AssertEquals ( 1000, demand.Quantity(10.00) );
			Assertion.AssertEquals ( 3500, demand.Quantity( 5.00) );  // -50% -> +250%
			Assertion.AssertEquals ( 1500, demand.Quantity( 9.00) );  // -10% -> +50%
			Assertion.AssertEquals (  500, demand.Quantity(11.00) );  // +10% -> -50%
			Assertion.AssertEquals (  250, demand.Quantity(11.50) );  // +15% -> -75%
			Assertion.AssertEquals (    0, demand.Quantity(12.00) );  // +20% -> -100%
			Assertion.AssertEquals (    0, demand.Quantity(15.00) );  // +50%	
		}

		[Test]
		public void TestFixedPrice() {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.KnownMarket(10.00,1000);
						
			Assertion.AssertEquals ( 10.00, demand.Price(10000) );
			Assertion.AssertEquals ( 10.00, demand.Price(  100) );
			Assertion.AssertEquals ( 10.00, demand.Price(    1) );
		}

		[Test]
		public void TestElasticPrice() {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.KnownMarket(10.00,1000);
			demand.FixedElasticity = 0;
						
			Assertion.AssertEquals ( 10.00, demand.Price(10000) );
			Assertion.AssertEquals ( 10.00, demand.Price(  100) );
			Assertion.AssertEquals ( 10.00, demand.Price(    1) );
		}

		[Test]
		public void TestInelasticPrice() {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.KnownMarket(10.00,1000);
			demand.FixedElasticity = -5;

			Assertion.AssertEquals (  -8.00, demand.Price(10000) ); // +900% -> -180%
			Assertion.AssertEquals (   0.00, demand.Price( 6000) ); // +500% -> -100%
			Assertion.AssertEquals (   2.00, demand.Price( 5000) ); // +400% -> -80%
			Assertion.AssertEquals (   4.00, demand.Price( 4000) ); // +300% -> -60%
			Assertion.AssertEquals (   8.00, demand.Price( 2000) ); // +100% -> -20%
			Assertion.AssertEquals (  10.00, demand.Price( 1000) ); // Known point
			Assertion.AssertEquals (  10.20, demand.Price(  900) ); // -10% -> +2%
			Assertion.AssertEquals (  11.00, demand.Price(  500) ); // -50% -> +10%
			Assertion.AssertEquals (  11.80, demand.Price(  100) ); // -90% -> +18%
			Assertion.AssertEquals (  11.98, demand.Price(   10) ); // -99% -> +19.8%
			Assertion.AssertEquals ( 11.998, demand.Price(    1) ); // -99.9% -> +19.98%
		}


		[Test]
		public void TestLinearPrice () {
			Demand demand = (Demand) LazyFactory.Create(typeof(Demand));
				
			demand.PotentialMarketSize = 1000000;
			demand.MaximumPrice = 100.00;

			Assertion.AssertEquals (   0.01, demand.Price(999900), PRICE_PRECISION );
			Assertion.AssertEquals (   1.00, demand.Price(990000), PRICE_PRECISION );
            Assertion.AssertEquals (  10.00, demand.Price(900000), PRICE_PRECISION );
			Assertion.AssertEquals (  50.00, demand.Price(500000), PRICE_PRECISION );
			Assertion.AssertEquals (  90.00, demand.Price(100000), PRICE_PRECISION );
			Assertion.AssertEquals (  99.00, demand.Price( 10000), PRICE_PRECISION );

		}

	}
}
